'*
'* Font Class
'*
'* Copyright(c) by Metlashevskiy Egor <br>
'*
'*

unit windows.gui.Font

$document "../../../doc/Font.htm"

import windows.gui.Control

published enum FontStyles
     fsBold = 1
     fsItalic = 2
     fsUnderline = 4
     fsStrikeOut = 8

end enum

published enum FontCharset
     fcAnsiCharset = 0
     fcDefaultCharset = 1
     fcSymbolCharset  = 2
     fcShiftjisCharset = 128
     fcHaNneulCharset = 129
     fcChinesebig5Charset = 136
     fcOemCharset = 255
end enum

published class Font: implements Control
private:
      dim m_font as long = 0
      dim m_fontname as string
public:
      dim LogFont as LOGFONT

      property hFont as long

published:
      property Width as integer
      property Height as integer
      property Escapement as integer
      property Orientation as integer
      property Weight as integer
      property Charset as FontCharset
      property Style as FontStyles
      property FontName as string
public:

      event OnChange ()

      declare sub Refresh ()

      sub Font ( byref Parent as Control )
end class

'default events
private sub Font_OnChange ( byref Sender as Font ): end sub

sub Font.Refresh ()
    if this.m_font <> null then
       DeleteObject ( this.m_font )
    end if
    this.m_font = CreateFontIndirect ( this.logfont )
end sub

'Font attributes
sub Font.Style ( byref Style as FontStyles )

if (Style and fsBold) then 
  logfont.lfWeight = 700
end if
if Style = 0 then
  logfont.lfWeight = 0
end if



logfont.lfItalic = ((Style and fsItalic) mod (fsItalic-1))
logfont.lfUnderline = ((Style and fsUnderline) mod (fsUnderline-1))
logfont.lfStrikeout = ((Style and fsStrikeout) mod (fsStrikeout-1))

    this.Refresh()
    this.OnChange()
end sub

function Font.Style () as FontStyles
dim mask as integer = 0

if logfont.lfWeight >= 700 then mask = mask or fsBold
if logfont.lfItalic then mask = mask or fsItalic
if logfont.lfUnderline then mask = mask or fsUnderline
if logfont.lfStrikeOut then mask = mask or fsStrikeOut

    return = mask

end function



sub Font.Width ( byref m_Value as integer )
    this.LogFont.lfWidth = m_Value
    this.Refresh()
    this.OnChange()
end sub

function Font.Width () as integer
    return = this.LogFont.lfWidth
end function

sub Font.Height ( byref m_Value as integer )
    this.LogFont.lfHeight = m_Value
    this.Refresh()
    this.OnChange()
end sub

function Font.Height () as integer
    return = this.LogFont.lfHeight
end function

sub Font.Escapement ( byref m_Value as integer )
    this.LogFont.lfEscapement = m_Value
    this.Refresh()
    this.OnChange()
end sub

function Font.Escapement () as integer
    return = this.LogFont.lfEscapement
end function

sub Font.Orientation ( byref m_Value as integer )
    this.LogFont.lfOrientation = m_Value
    this.Refresh()
    this.OnChange()
end sub

function Font.Orientation () as integer
    return = this.LogFont.lfEscapement
end function

sub Font.FontName ( byref m_Value as string )
    this.m_FontName = m_Value
    strcpy ( ref this.logfont.lfFaceName, this.m_FontName)
    this.Refresh()
    this.OnChange()
end sub

function Font.FontName () as string
    return = this.m_FontName
end function

sub Font.Weight ( byref m_Value as integer )
    this.LogFont.lfWeight = m_Value
    this.Refresh()
    this.OnChange()
end sub

function Font.Weight () as integer
    return = this.LogFont.lfWeight
end function

sub Font.Charset ( byref m_Value as FontCharset )
    this.LogFont.lfCharset = m_Value
    this.Refresh()
    this.OnChange()
end sub

function Font.Charset () as FontCharset
    return = this.LogFont.lfCharset
end function


sub Font.hFont ( byref m_hFont as long )
    if this.m_font <> null then
       DeleteObject ( this.m_font )
    end if
    this.m_font = m_hFont
end sub

function Font.hFont () as long
    return = this.m_font
end function

sub Font.Font ( byref Parent as Control )

    this.logfont.lfHeight = 15
    this.logfont.lfWidth = 0
    this.logfont.lfEscapement = 0
    this.logfont.lfOrientation = 0
    this.logfont.lfWeight = 0
    this.logfont.lfItalic = 0
    this.logfont.lfUnderline = 0
    this.logfont.lfStrikeOut = 0
    this.logfont.lfCharSet = 0
    this.logfont.lfOutPrecision = 0
    this.logfont.lfClipPrecision = 0
    this.logfont.lfQuality = 0
    this.logfont.lfPitchAndFamily = 0
    this.m_FontName = "MS Sans Serif"
    strcpy ( ref this.logfont.lfFaceName, this.m_FontName)

    this.Refresh()

    this.ParentControl = Parent
    this.OnChange = subptr ( Font_OnChange )

end sub



sub InitLibrary

    RegisterClass ( "Font", subptrclass ( Font, Font ) )

end sub
